@@ -20,9 +20,11 @@ rollmean.zoo <- function(x, k, fill = if (na.pad) NA, na.pad = FALSE,
 
   if (length(dim(x)) == 2) {
 	  # merge is the only zoo specific part of this method
+	  
 	  out <- do.call("merge", c(lapply(1:NCOL(x), function(i) {
 		rollmean(x[, i, drop = TRUE], k, fill = fill, align = align, ...)
 	  }), all = FALSE))
+	  if (ncol(x) == 1) dim(out) <- c(length(out), 1)
 	  colnames(out) <- colnames(x)
 	  return(out)
   }
@@ -56,6 +58,62 @@ rollmean.ts <- function(x, k, fill = if (na.pad) NA, na.pad = FALSE,
 		as.ts(rollmean(as.zoo(x), k, fill = fill, align = align, ...))
 }
 
+rollsumr <- function(..., align = "right") {
+	rollsum(..., align = align)
+}
+
+rollsum <- function(x, k, fill = if (na.pad) NA, na.pad = FALSE, 
+	align = c("center", "left", "right"), ...) {
+	UseMethod("rollsum")
+}
+
+rollsum.zoo <- function(x, k, fill = if (na.pad) NA, na.pad = FALSE, 
+	align = c("center", "left", "right"), ...) {
+
+  if (!missing(na.pad)) warning("na.pad is deprecated. Use fill.")
+
+  align <- match.arg(align)
+
+  if (length(dim(x)) == 2) {
+	  # merge is the only zoo specific part of this method
+	  
+	  out <- do.call("merge", c(lapply(1:NCOL(x), function(i) {
+		rollsum(x[, i, drop = TRUE], k, fill = fill, align = align, ...)
+	  }), all = FALSE))
+	  if (ncol(x) == 1) dim(out) <- c(length(out), 1)
+	  colnames(out) <- colnames(x)
+	  return(out)
+  }
+
+  n <- length(x)
+  stopifnot(k <= n)
+
+  ix <- switch(align,
+      "left" = { 1:(n-k+1) },
+      "center" = { floor((1+k)/2):ceiling(n-k/2) },
+      "right" = { k:n })
+
+  xu <- unclass(x)
+  y <- xu[k:n] - xu[c(1, seq_len(n-k))] # difference from previous
+  y[1] <- sum(xu[1:k])		 # find the first
+  # sum precomputed differences
+  rval <- cumsum(y)
+
+  x[ix] <- rval
+  na.fill(x, fill = fill, ix)
+
+}
+
+rollsum.default <- function(x, k, fill = if (na.pad) NA, na.pad = FALSE, 
+	align = c("center", "left", "right"), ...) {
+		coredata(rollsum(zoo(x), k, fill = fill, align = align, ...))
+}
+
+rollsum.ts <- function(x, k, fill = if (na.pad) NA, na.pad = FALSE, 
+	align = c("center", "left", "right"), ...) {
+		as.ts(rollsum(as.zoo(x), k, fill = fill, align = align, ...))
+}
+
 rollmaxr <- function(..., align = "right") {
 	rollmax(..., align = align)
 }
@@ -77,7 +135,8 @@ rollmax.zoo <- function(x, k, fill = if (na.pad) NA, na.pad = FALSE,
 	  out <- do.call("merge", c(lapply(1:NCOL(x), function(i) {
 		rollmax(x[, i, drop = TRUE], k, fill = fill, align = align, ...)
 	  }), all = FALSE))
-	  colnames(out) <- colnames(x)
+	  if (ncol(x) == 1) dim(out) <- c(length(out), 1)
+	  colnames(out) <- if (ncol(x) == ncol(out)) colnames(x)
 	  return(out)
   }
 
@@ -139,6 +198,7 @@ rollmedian.zoo <- function(x, k, fill = if (na.pad) NA, na.pad = FALSE,
 	  out <- do.call("merge", c(lapply(1:NCOL(x), function(i) {
 		rollmedian(x[, i, drop = TRUE], k, fill = fill, align = align, ...)
 	  }), all = FALSE))
+	  if (ncol(x) == 1) dim(out) <- c(length(out), 1)
 	  colnames(out) <- colnames(x)
 	  return(out)
   }
@@ -154,7 +214,7 @@ rollmedian.zoo <- function(x, k, fill = if (na.pad) NA, na.pad = FALSE,
   m <- k %/% 2
   rval <- runmed(x, k, ...)
   attr(rval, "k") <- NULL
-  rval <- rval[-c(1:m, (n-m+1):n)]
+  if(m >= 1) rval <- rval[-c(1:m, (n-m+1):n)]
 
   x[ix] <- rval
   na.fill(x, fill = fill, ix)
